
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class LearningObjectSerializer < ActiveModel::Serializer
  cache key: 'learning_object', expires_in: 4.hours, except: [:likes_count, :liked, :reviewed, :complained, :views_count, :downloads_count, :review_average, :submission_id]

  def default_mime_type
    object.default_attachment.try(:mime_type)
  end

  def default_attachment_id
    object.default_attachment.try(:id)
  end

  def thumbnail
    object.default_thumbnail
  end

  def object_type
    object.object_type.try(:name)
  end

  def liked
    object.liked? current_user
  end

  def reviewed
    object.reviewed? current_user
  end

  def complained
    object.complained? current_user
  end

  def submission_id
    object.submission.try(:id) if object.submitted? && current_user.is_curator?
  end

  attributes \
    :id,
    :name,
    :description,
    :author,
    :curator,
    :thumbnail,
    :object_type,
    :language,
    :default_attachment_location,
    :default_attachment_id,
    :submission_id,
    :default_mime_type,
    :score,
    :state,
    :review_average,
    :link,
    :software,
    :license,
    :liked,
    :likes_count,
    :downloads_count,
    :views_count,
    :reviewed,
    :complained,
    :shares_count,
    :created_at,
    :published_at,
    :terms_of_service,
    :updated_at,
    :magnetlink

  belongs_to :publisher
  has_many :tags
  has_many :subjects
  has_many :educational_stages
  has_many :attachments
end
