
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class ScoreCalculatorService
  def initialize(object)
    @object = object
    @class = @object.class.name
    @value = nil
  end

  def calculate(values = nil)
    values = object_hash if values.nil?
    values = values.with_indifferent_access

    sum = @object.is_a?(User) ? calculate_user(values) : calculate_object(values)
    sum_div = sum.to_f / sum_weights.to_f
    sum_div = sum_div.nan? ? 0.0 : sum_div

    @value = sum_div.round(6)
  end

  def category_by_reputation
    return nil if @value.nil?

    UserCategory.order(:reference).each do |uc|
      next if @value > uc.reference
      return uc.id
    end
  end

  private

  def object_hash
    Score.object_type(@object.class.name).map { |s| [s.code, @object.try(s.code)] }.to_h
  end

  def sum_weights
    Score.object_type(@object.class.name).pluck(:weight).inject(0.0) { |acc, elem| acc + elem }
  end

  def calculate_user(values)
    sum = 0.0
    Score.object_type(@object.class.name).find_each do |score|
      max = score.max_value.to_f
      next if max.zero?

      sum += (values[score.code].to_f / max) * score.weight.to_f
    end
    sum
  end

  def calculate_object(values)
    sum = 0.0
    Score.object_type(@object.class.name).find_each do |score|
      sum += values[score.code].to_f * score.weight.to_f
    end
    sum
  end
end
