
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class AttachmentCacheWorker
  include Sidekiq::Worker

  sidekiq_options queue: :attachment_cache

  def perform(id, directory, force = false)
    @attachment = LearningObject::Attachment.where(id: id).first
    client = DspaceService.create_client
    return false if @attachment.nil? || client.nil?

    @directory = directory
    file_root = AttachmentCacheService.new(@attachment).file_root

    return @attachment.update(cache_link: cache_link) if File.exist?(file_root) && !force

    file = client.bitstreams.retrieve(id: @attachment.id_dspace).open
    begin
      FileUtils.mkdir_p(directory_root)
      FileUtils.mv(file.path, file_root)
      FileUtils.chmod 0644, file_root.to_s, verbose: true
      FileUtils.chown_R 'portalmec', 'portalmec', directory_root.to_s, verbose: true
      @attachment.update(cache_link: cache_link)
    ensure
      file.close if !file.nil? && File.exist?(file.path)
    end
  ensure
    ActiveRecord::Base.clear_active_connections!
    client.close_connection unless client.nil?
  end

  private

  def cache_link
    "/#{file_path}"
  end

  def file_path
    "#{@directory}/#{SanitizationService.filename(@attachment.name)}"
  end

  def directory_root
    Rails.root.join('public', @directory)
  end
end
